@@ -5,5 +5,10 @@
 	.Call( "check_libprotobuf_version", minversion, PACKAGE = "RProtoBuf" )
 	readProtoFiles( package = pkgname )
 	attachDescriptorPool( pos = length(search()) )
+	
+	if( exists( ".httpd.handlers.env", asNamespace( "tools" ) ) ){
+		e <- tools:::.httpd.handlers.env
+		e[["RProtoBuf"]] <- RProtoBuf.http.handler
+	}
 }
 
